// eslint-disable-next-line @typescript-eslint/no-explicit-any
export const buildRoutePath = (map, name, params = {}) => {
    const route = map.get(name);
    if (!route) {
        throw new Error(`Undefined route "${name}"`);
    }
    if (!route.path) {
        throw new Error(`Route "${name}" does not have a path`);
    }
    return route.path
        .replace(/:\w+\??/g, (match) => {
        const paramKey = match.replace(/[:?]/g, '');
        const paramValue = params[paramKey];
        if (!paramValue && !match.includes('?')) {
            throw new Error(`Missing value for required param "${paramKey}"`);
        }
        return paramValue || '';
    })
        .replace(/\/$/, '');
};
export const mapRoutes = (routes, parents = [], map = new Map()) => {
    routes.forEach((route) => {
        if (route.routes) {
            mapRoutes(route.routes, route.name ? [...parents, route.name] : parents, map);
        }
        if (!route.name && route.path) {
            throw new Error(`Route ${route.path} has no name`);
        }
        if (!route.name) {
            return;
        }
        const existingRoute = map.get(route.name);
        if (process.env.NODE_ENV !== 'production' && existingRoute) {
            // eslint-disable-next-line no-console
            console.warn(`Duplicate definition for route ${route.name}:\n- ${existingRoute.path}\n- ${route.path}`);
        }
        map.set(route.name, Object.assign({}, route, { parents }, (route.path ? {
            regex: new RegExp(`^${route.path.replace(/\/:\w+\?/g, '/?([\\w_-]*)').replace(/:\w+/g, '([\\w_-]+)')}\\/?$`),
        } : {})));
    });
    return map;
};
export class BaseRoutingContext {
    constructor(routesMap) {
        this.match = (pathname) => Array.from(this.routesMap.values())
            .find(route => !!route.regex && route.regex.test(pathname)) || null;
        this.getPath = (name, params) => buildRoutePath(this.routesMap, name, params);
        this.getRoute = (name) => {
            const route = this.routesMap.get(name);
            if (!route) {
                throw new Error(`Undefined route "${name}"`);
            }
            return route;
        };
        this.routesMap = routesMap;
    }
}
export class RoutingContext extends BaseRoutingContext {
    constructor(routesMap, history) {
        super(routesMap);
        this.push = (name, params) => this.history
            .push(buildRoutePath(this.routesMap, name, params));
        this.replace = (name, params) => this.history
            .replace(buildRoutePath(this.routesMap, name, params));
        this.history = history;
    }
}
export function buildRoutingContext(routes, history) {
    const map = mapRoutes(routes);
    if (!history) {
        return new BaseRoutingContext(map);
    }
    return new RoutingContext(map, history);
}
/* eslint-enable import/export */
